import io
import logging
import os
import shutil
import zipfile
from datetime import date
from pathlib import Path

import pandas as pd
import requests

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


PROJECT_ROOT = Path(__file__).parent.parent.parent
RESOURCE_DIR = os.path.join(PROJECT_ROOT, "resources")
TODAY = date.today().strftime("%Y%m%d")


class Download:
    """
    Download LEI to ISIN mappings
    """

    def __init__(self, _is_actions=False):
        """
        Downloads LEI ISIN mapping 

        Args:
            _is_actions (bool): For setting path of downloaded resources on Github Actions
        """
        self.data_url = f"https://isinmapping.gleif.org/file-by-date/{TODAY}"

        self._download(_is_actions)

    def _download(self, _is_actions):
        """
        Initiate download into resource folder
        """
        if not os.path.exists(RESOURCE_DIR):
            logger.info(f"No resources directory found, creating resources directory.")
            os.mkdir(RESOURCE_DIR)

        try:
            response = requests.get(self.data_url)
        except requests.exceptions as err:
            logger.error(
                "Connection Error, Unable to download data at this time. Please check you have working internet connection or try again later."
            )
        if not response.ok:
            logger.error("No response from GLEIF server.")

        logger.info("The file could be over 50 Mb.")
        # TODO: Add progress bar
        zipped_content = zipfile.ZipFile(io.BytesIO(response.content))
        # TODO: Remove this
        if _is_actions:
            zipped_content.extractall(
                "/home/runner/work/python-lei/python-lei/resources"
            )
        else:
            zipped_content.extractall(RESOURCE_DIR)
            logger.info(f"Extraction complete in {RESOURCE_DIR}")

    # TODO: Covert the dataframe to parquet and use it.


class Update:
    """
    Update the ISIN mapping in the resource folder
    """

    def __init__(self):
        """
        Update the ISIN LEI data
        """
        # Remove the old data and call the download class

        if (
            not os.path.exists(RESOURCE_DIR) or os.listdir(RESOURCE_DIR) == []
        ):  # TODO: use not
            logger.info(
                "Resource directory not found or LEI ISIN mappings not found. Downloading now."
            )
            download = Download()

        if os.listdir(RESOURCE_DIR) != []:
            shutil.rmtree(RESOURCE_DIR)
            logger.info(f"Downloading Data in {RESOURCE_DIR}")
            download = Download()


def load_data():
    """
    Loads and Returns the dataframe
    """
    dataframe = pd.read_csv(os.path.join(RESOURCE_DIR, os.listdir(RESOURCE_DIR)[0]))
    return dataframe



import requests
from python_lei.exceptions import NotFound
import logging
from texttable import Texttable

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class SearchLEI:
    """
    Search LEI number from name
    """

    def __init__(self):
        self.search_url = "https://api.leilex.com/API/LEI/AutoComplete/"

    def search_lei(self, company_name, show_table=False):
        """
        Search LEI numbers with company name

        Args:
            company_name (str): Name of the company to search LEI numbers for
        
        Returns:
            raw_output (List): Raw output from the API

        """
        if not isinstance(company_name, str):
            raise ValueError("Company Name should be a string")

        if not company_name:
            raise NotFound("Company Name not found. Please provide a company name")

        raw_output = self._request_api(company_name)

        if not raw_output:
            logger.info(f"No LEI found for company name {company_name}")
            return

        if not show_table:
            return raw_output

        # Initalize the table
        table = Texttable()
        table.add_row(["Legal Name", "LEI"])

        for legal_entity in raw_output:
            table.add_row([legal_entity.get("LegalName"), legal_entity.get("LEI")])

        return raw_output, table.draw()

    def _request_api(self, company_name):
        """
        Request LEI autocomplete api to obtain possible list of LEI numbers
        """
        try:
            response = requests.get(
                f"{self.search_url}?query={company_name}&filterType=Name"
            )

            if not response.ok:
                logger.info("No response form the API")
                return
            return response.json()
        except requests.exceptions.ConnectionError as err:
            logger.error("Error connecting to API host", exc_info=err)
            return



from python_lei.exceptions import RecordNotFound


class LEIParser:
    """
    Helper class to parse LEI response returned from the API
    """

    def __init__(self, lei_responses):
        """
        Get the output from the API and assign them to variables

        Args:
            lei_responses: List of raw LEI reponse from the API
        """
        # Initialize empty lists
        self.lei_list = []
        self.lei_names = []
        self.lei_legal_jurisdictions = []
        self.lei_legal_forms = []
        self.lei_other_legal_forms = []
        self.lei_entity_status = []
        self.lei_entity_expiration_dates = []
        self.lei_entity_expiration_reasons = []
        self.lei_successor_entities = []
        self.lei_initial_registration_dates = []
        self.lei_last_update_dates = []
        self.lei_registration_status = []
        self.lei_next_renewal_dates = []
        self.lei_managing_LOU = []
        self.lei_validation_sources = []
        self.lei_associated_lei = []
        self.lei_associated_entity_names = []
        self.lei_associated_entity_types = []
        self.lei_registration_authority_ids = []
        self.lei_other_registration_ids = []
        self.lei_registration_authority_entite_ids = []
        self.lei_entity_categories = []
        self.lei_address_1_line_1 = []
        self.lei_address_1_line_2 = []
        self.lei_address_1_line_3 = []
        self.lei_address_1_line_4 = []
        self.lei_address_1_city = []
        self.lei_address_1_region = []
        self.lei_address_1_country = []
        self.lei_address_1_postal_code = []
        self.lei_address_1_other_type = []
        self.lei_address_1_address_type = []
        self.lei_address_2_line_1 = []
        self.lei_address_2_line_2 = []
        self.lei_address_2_line_3 = []
        self.lei_address_2_line_4 = []
        self.lei_address_2_city = []
        self.lei_address_2_region = []
        self.lei_address_2_country = []
        self.lei_address_2_postal_code = []
        self.lei_address_2_other_type = []
        self.lei_address_2_address_type = []
        self.lei_other_name_name = []
        self.lei_other_name_language = []
        self.lei_other_name_type = []
        self.lei_reporting_exception_1_lei = []
        self.lei_reporting_exception_1_category = []
        self.lei_reporting_exception_2_lei = []
        self.lei_reporting_exception_2_category = []
        self.lei_relationships_1_parent_lei = []
        self.lei_relationships_1_parent_relationship_type = []
        self.lei_relationships_1_parent_relationship_status = []
        self.lei_relationships_1_parent_initial_registration_date = []
        self.lei_relationships_1_parent_last_updated = []
        self.lei_relationships_1_parent_registration_status = []
        self.lei_relationships_1_parent_next_renewal_date = []
        self.lei_relationships_1_parent_managing_LOU = []
        self.lei_relationships_1_parent_validation_sources = []
        self.lei_relationships_1_parent_validation_documents = []
        self.lei_relationships_1_parent_validation_reference = []
        self.lei_relationships_2_parent_lei = []
        self.lei_relationships_2_parent_relationship_type = []
        self.lei_relationships_2_parent_relationship_status = []
        self.lei_relationships_2_parent_initial_registration_date = []
        self.lei_relationships_2_parent_last_updated = []
        self.lei_relationships_2_parent_registration_status = []
        self.lei_relationships_2_parent_next_renewal_date = []
        self.lei_relationships_2_parent_managing_LOU = []
        self.lei_relationships_2_parent_validation_sources = []
        self.lei_relationships_2_parent_validation_documents = []
        self.lei_relationships_2_parent_validation_reference = []

        if not lei_responses:
            raise IndexError("No LEI found")

        for lei_response in lei_responses:
            if not lei_response.get("records"):
                raise RecordNotFound("No record found")
                continue
            self.lei_list.append(lei_response["records"][0].get("LEI"))
            self.lei_names.append(lei_response["records"][0].get("LegalName"))
            self.lei_legal_jurisdictions.append(
                lei_response["records"][0].get("LegalJurisdiction")
            )
            self.lei_legal_forms.append(lei_response["records"][0].get("LegalForm"))
            self.lei_other_legal_forms.append(
                lei_response["records"][0].get("OtherLegalForm")
            )
            self.lei_entity_status.append(
                lei_response["records"][0].get("EntityStatus")
            )
            self.lei_entity_expiration_dates.append(
                lei_response["records"][0].get("EntityExpirationDate")
            )
            self.lei_entity_expiration_reasons.append(
                lei_response["records"][0].get("EntityExpirationReason")
            )
            self.lei_successor_entities.append(
                lei_response["records"][0].get("SuccessorEntity")
            )
            self.lei_initial_registration_dates.append(
                lei_response["records"][0].get("InitialRegistrationDate")
            )
            self.lei_last_update_dates.append(
                lei_response["records"][0].get("LastUpdateDate")
            )
            self.lei_registration_status.append(
                lei_response["records"][0].get("RegistrationStatus")
            )
            self.lei_next_renewal_dates.append(
                lei_response["records"][0].get("NextRenewalDate")
            )
            self.lei_managing_LOU.append(lei_response["records"][0].get("ManagingLOU"))
            self.lei_validation_sources.append(
                lei_response["records"][0].get("ValidationSources")
            )
            self.lei_associated_lei.append(
                lei_response["records"][0].get("AssociatedLEI")
            )
            self.lei_associated_entity_names.append(
                lei_response["records"][0].get("AssociatedEntityName")
            )
            self.lei_associated_entity_types.append(
                lei_response["records"][0].get("AssociatedEntityType")
            )
            self.lei_registration_authority_ids.append(
                lei_response["records"][0].get("RegistrationAuthorityID")
            )
            self.lei_other_registration_ids.append(
                lei_response["records"][0].get("OtherRegistrationAuthorityID")
            )
            self.lei_registration_authority_entite_ids.append(
                lei_response["records"][0].get("RegistrationAuthorityEntityID")
            )
            self.lei_entity_categories.append(
                lei_response["records"][0].get("EntityCategory")
            )
            self.lei_address_1_line_1.append(
                lei_response["records"][0].get("Addresses")[0].get("Line1")
            )
            self.lei_address_1_line_2.append(
                lei_response["records"][0].get("Addresses")[0].get("Line2")
            )
            self.lei_address_1_line_3.append(
                lei_response["records"][0].get("Addresses")[0].get("Line3")
            )
            self.lei_address_1_line_4.append(
                lei_response["records"][0].get("Addresses")[0].get("Line4")
            )
            self.lei_address_1_city.append(
                lei_response["records"][0].get("Addresses")[0].get("City")
            )
            self.lei_address_1_region.append(
                lei_response["records"][0].get("Addresses")[0].get("Region")
            )
            self.lei_address_1_country.append(
                lei_response["records"][0].get("Addresses")[0].get("Country")
            )
            self.lei_address_1_postal_code.append(
                lei_response["records"][0].get("Addresses")[0].get("PostalCode")
            )
            self.lei_address_1_other_type.append(
                lei_response["records"][0].get("Addresses")[0].get("OtherType")
            )
            self.lei_address_1_address_type.append(
                lei_response["records"][0].get("Addresses")[0].get("AddressType")
            )
            self.lei_address_2_line_1.append(
                lei_response["records"][0].get("Addresses")[1].get("Line1")
            )
            self.lei_address_2_line_2.append(
                lei_response["records"][0].get("Addresses")[1].get("Line2")
            )
            self.lei_address_2_line_3.append(
                lei_response["records"][0].get("Addresses")[1].get("Line3")
            )
            self.lei_address_2_line_4.append(
                lei_response["records"][0].get("Addresses")[1].get("Line4")
            )
            self.lei_address_2_city.append(
                lei_response["records"][0].get("Addresses")[1].get("City")
            )
            self.lei_address_2_region.append(
                lei_response["records"][0].get("Addresses")[1].get("Region")
            )
            self.lei_address_2_country.append(
                lei_response["records"][0].get("Addresses")[1].get("Country")
            )
            self.lei_address_2_postal_code.append(
                lei_response["records"][0].get("Addresses")[1].get("PostalCode")
            )
            self.lei_address_2_other_type.append(
                lei_response["records"][0].get("Addresses")[1].get("OtherType")
            )
            self.lei_address_2_address_type.append(
                lei_response["records"][0].get("Addresses")[1].get("AddressType")
            )
            if lei_response["records"][0].get("OtherNames"):
                self.lei_other_name_name.append(
                    lei_response["records"][0].get("OtherNames")[0].get("Name")
                )
                self.lei_other_name_language.append(
                    lei_response["records"][0].get("OtherNames")[0].get("LanguageType")
                )
                self.lei_other_name_type.append(
                    lei_response["records"][0].get("OtherNames")[0].get("NameType")
                )
            else:
                self.lei_other_name_name.append(None)
                self.lei_other_name_language.append(None)
                self.lei_other_name_type.append(None)

            if lei_response["records"][0].get("ReportingExceptions"):
                self.lei_reporting_exception_1_lei.append(
                    lei_response["records"][0].get("ReportingExceptions")[0].get("LEI")
                )
                self.lei_reporting_exception_1_category.append(
                    lei_response["records"][0]
                    .get("ReportingExceptions")[0]
                    .get("ExceptionCategory")
                )
                self.lei_reporting_exception_2_lei.append(
                    lei_response["records"][0].get("ReportingExceptions")[1].get("LEI")
                )
                self.lei_reporting_exception_2_category.append(
                    lei_response["records"][0]
                    .get("ReportingExceptions")[1]
                    .get("ExceptionCategory")
                )
            else:
                self.lei_reporting_exception_1_lei.append(None)
                self.lei_reporting_exception_1_category.append(None)
                self.lei_reporting_exception_2_lei.append(None)
                self.lei_reporting_exception_2_category.append(None)
            if lei_response["records"][0].get("Relationships"):
                self.lei_relationships_1_parent_lei.append(
                    lei_response["records"][0].get("Relationships")[0].get("ParentLEI")
                )
                self.lei_relationships_1_parent_relationship_type.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("RelationshipType")
                )
                self.lei_relationships_1_parent_relationship_status.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("RelationshipStatus")
                )
                self.lei_relationships_1_parent_initial_registration_date.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("InitialRegistrationDate")
                )
                self.lei_relationships_1_parent_last_updated.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("LastUpdateDate")
                )
                self.lei_relationships_1_parent_registration_status.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("RegistrationStatus")
                )
                self.lei_relationships_1_parent_next_renewal_date.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("NextRenewalDate")
                )
                self.lei_relationships_1_parent_managing_LOU.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("ManagingLou")
                )
                self.lei_relationships_1_parent_validation_sources.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("ValidationSources")
                )
                self.lei_relationships_1_parent_validation_documents.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("ValidationDocuments")
                )
                self.lei_relationships_1_parent_validation_reference.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("ValidationReference")
                )
                self.lei_relationships_2_parent_lei.append(
                    lei_response["records"][0].get("Relationships")[0].get("ParentLEI")
                )
                self.lei_relationships_2_parent_relationship_type.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("RelationshipType")
                )
                self.lei_relationships_2_parent_relationship_status.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("RelationshipStatus")
                )
                self.lei_relationships_2_parent_initial_registration_date.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("InitialRegistrationDate")
                )
                self.lei_relationships_2_parent_last_updated.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("LastUpdateDate")
                )
                self.lei_relationships_2_parent_registration_status.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("RegistrationStatus")
                )
                self.lei_relationships_2_parent_next_renewal_date.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("NextRenewalDate")
                )
                self.lei_relationships_2_parent_managing_LOU.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("ManagingLou")
                )
                self.lei_relationships_2_parent_validation_sources.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("ValidationSources")
                )
                self.lei_relationships_2_parent_validation_documents.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("ValidationDocuments")
                )
                self.lei_relationships_2_parent_validation_reference.append(
                    lei_response["records"][0]
                    .get("Relationships")[0]
                    .get("ValidationReference")
                )
            else:
                self.lei_relationships_1_parent_lei.append(None)
                self.lei_relationships_1_parent_relationship_type.append(None)
                self.lei_relationships_1_parent_relationship_status.append(None)
                self.lei_relationships_1_parent_initial_registration_date.append(None)
                self.lei_relationships_1_parent_last_updated.append(None)
                self.lei_relationships_1_parent_registration_status.append(None)
                self.lei_relationships_1_parent_next_renewal_date.append(None)
                self.lei_relationships_1_parent_managing_LOU.append(None)
                self.lei_relationships_1_parent_validation_sources.append(None)
                self.lei_relationships_1_parent_validation_documents.append(None)
                self.lei_relationships_1_parent_validation_reference.append(None)
                self.lei_relationships_2_parent_lei.append(None)
                self.lei_relationships_2_parent_relationship_type.append(None)
                self.lei_relationships_2_parent_relationship_status.append(None)
                self.lei_relationships_2_parent_initial_registration_date.append(None)
                self.lei_relationships_2_parent_last_updated.append(None)
                self.lei_relationships_2_parent_registration_status.append(None)
                self.lei_relationships_2_parent_next_renewal_date.append(None)
                self.lei_relationships_2_parent_managing_LOU.append(None)
                self.lei_relationships_2_parent_validation_sources.append(None)
                self.lei_relationships_2_parent_validation_documents.append(None)
                self.lei_relationships_2_parent_validation_reference.append(None)



# -*- coding: utf-8 -*-
import logging

from pkg_resources import DistributionNotFound, get_distribution
from python_lei import pylei

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = "python-lei"
    __version__ = get_distribution(dist_name).version
except DistributionNotFound:
    __version__ = "unknown"
finally:
    del get_distribution, DistributionNotFound


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()



import pandas as pd
import requests
from python_lei.exceptions import InvalidISIN, InvalidLEI
from python_lei.utils import load_data


class LEItoISIN:
    """
    Get ISIN from corresponding LEI
    """

    def __init__(self):
        self.dataframe = load_data()

    def get_isin(self, lei, return_dataframe=False):
        """
        Get all the possible ISIN numbers for a LEI
        """
        if len(lei) != 20:
            raise InvalidLEI("Invalid LEI number")

        isin_dataframe = self.dataframe[self.dataframe["LEI"] == lei]

        isin_list = isin_dataframe["ISIN"].tolist()

        if return_dataframe:
            return isin_list, isin_dataframe

        return isin_list


class ISINtoLEI:
    """
    Get LEI from corresponding ISIN
    """

    def __init__(self):
        self.dataframe = load_data()

    def get_lei(self, isin):
        """
        Get all the possible ISIN numbers for a LEI
        """
        if len(isin) != 12:
            raise InvalidISIN("Invalid ISIN number")

        lei_dataframe = self.dataframe[self.dataframe["ISIN"] == isin]

        lei_list = lei_dataframe["LEI"].tolist()

        return lei_list



import logging

import pandas as pd
import requests
from dateutil.parser import parse
from python_lei.exceptions import InvalidLEI
from python_lei.parser import LEIParser

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class pyLEI:
    """
    Main class to handle lei requests and return data
    """

    def __init__(self):
        """
        Initialize API URL
        """
        self.api_url = "https://api.leilex.com/API/LEI/"

    def get_lei_info(self, lei_list, return_dataframe=False):
        """
        Request API and return response
        
        Args:
            lei_list: List of LEIs
            return_dataframe: Whether to return a dataframe or not.
        
        Returns:
            raw_output: List of Dicts with raw API json response
            lei_results: LEI result as class
            dataframe: Pandas dataframe with results.
        """
        if not isinstance(lei_list, list):
            raise AttributeError("Invalid input, please provide LEI in a list")

        if any(lei for lei in lei_list if len(lei) != 20):
            raise InvalidLEI("Invalid LEI number found in the list")

        if len(lei_list) > 20:
            raise ValueError(
                "To respect the free API request quota we have limited the current LEI numbers to 20"
            )

        raw_output = self._request_api(lei_list)

        if not raw_output:
            if return_dataframe:
                return None, None, None
            else:
                return None, None

        lei_results = LEIParser(raw_output)
        if return_dataframe:
            dataframe = pd.DataFrame(
                {
                    "LEI": lei_results.lei_list,
                    "Legal_Name": lei_results.lei_names,
                    "Legal_Jurisdiction": lei_results.lei_legal_jurisdictions,
                    "LegalForm": lei_results.lei_legal_forms,
                    "Other_Legal_Form": lei_results.lei_other_legal_forms,
                    "Entity_Status": lei_results.lei_entity_status,
                    "Entity_Expiration_Date": lei_results.lei_entity_expiration_dates,
                    "Entity_Expiration_Reason": lei_results.lei_entity_expiration_reasons,
                    "Successor_Entity": lei_results.lei_successor_entities,
                    "InitialRegistrationDate": [
                        parse(date)
                        if date else None for date in lei_results.lei_initial_registration_dates
                    ],
                    "Last_Update_Date": [
                        parse(date) if date else None for date in lei_results.lei_last_update_dates
                    ],
                    "Registration_Status": lei_results.lei_registration_status,
                    "Next_Renewal_Date": lei_results.lei_next_renewal_dates,
                    "Managing_LOU": lei_results.lei_managing_LOU,
                    "Validation_Sources": lei_results.lei_validation_sources,
                    "Associated_LEI": lei_results.lei_associated_lei,
                    "Associated_Entity_Name": lei_results.lei_associated_entity_names,
                    "Associated_Entity_Type": lei_results.lei_associated_entity_types,
                    "Registration_Authority_ID": lei_results.lei_registration_authority_ids,
                    "Registration_Authority_Entity_ID": lei_results.lei_registration_authority_entite_ids,
                    "Entity_Category": lei_results.lei_entity_categories,
                    "Address_1_Line_1": lei_results.lei_address_1_line_1,
                    "Address_1_Line_2": lei_results.lei_address_1_line_2,
                    "Address_1_Line_3": lei_results.lei_address_1_line_3,
                    "Address_1_Line_4": lei_results.lei_address_1_line_4,
                    "Address_1_city": lei_results.lei_address_1_city,
                    "Address_1_region": lei_results.lei_address_1_region,
                    "Address_1_country": lei_results.lei_address_1_country,
                    "Address_1_postal_code": lei_results.lei_address_1_postal_code,
                    "Address_1_type": lei_results.lei_address_2_address_type,
                    "Address_2_Line_1": lei_results.lei_address_2_line_1,
                    "Address_2_Line_2": lei_results.lei_address_2_line_2,
                    "Address_2_Line_3": lei_results.lei_address_2_line_3,
                    "Address_2_Line_4": lei_results.lei_address_2_line_4,
                    "Address_2_city": lei_results.lei_address_2_city,
                    "Address_2_region": lei_results.lei_address_2_region,
                    "Address_2_country": lei_results.lei_address_2_country,
                    "Address_2_postal_code": lei_results.lei_address_2_postal_code,
                    "Address_2_type": lei_results.lei_address_2_address_type,
                    "Other_Name": lei_results.lei_other_name_name,
                    "Other_Name_Language": lei_results.lei_other_name_language,
                    "Other_Name_Type": lei_results.lei_other_name_type,
                    "Relationship_1_Patrent_LEI": lei_results.lei_relationships_1_parent_lei,
                    "Relationship_1_Relationship_Type": lei_results.lei_relationships_1_parent_relationship_type,
                    "Relationship_1_Relationship_Status": lei_results.lei_relationships_1_parent_relationship_status,
                    "Relationship_1_Initial_Registration_Date": [
                        parse(date)
                        if date else None for date in lei_results.lei_relationships_1_parent_initial_registration_date
                    ],
                    "Relationship_1_Last_Update_Date": [
                        parse(date)
                        if date else None for date in lei_results.lei_relationships_1_parent_last_updated
                    ],
                    "Relationship_1_Registration_Status": lei_results.lei_relationships_1_parent_registration_status,
                    "Relationship_1_Next_Renewal_Date": [
                        parse(date)
                        if date else None for date in lei_results.lei_relationships_1_parent_next_renewal_date
                    ],
                    "Relationship_1_Managing_Lou": lei_results.lei_relationships_1_parent_managing_LOU,
                    "Relationship_1_Validation_Sources": lei_results.lei_relationships_1_parent_validation_sources,
                    "Relationship_1_Validation_Documents": lei_results.lei_relationships_1_parent_validation_documents,
                    "Relationship_1_Validation_Reference": lei_results.lei_relationships_1_parent_validation_reference,
                    "Relationship_2_Patrent_LEI": lei_results.lei_relationships_2_parent_lei,
                    "Relationship_2_Relationship_Type": lei_results.lei_relationships_2_parent_relationship_type,
                    "Relationship_2_Relationship_Status": lei_results.lei_relationships_2_parent_relationship_status,
                    "Relationship_2_Initial_Registration_Date": [
                        parse(date)
                        if date else None for date in lei_results.lei_relationships_2_parent_initial_registration_date
                    ],
                    "Relationship_2_Last_Update_Date": [
                        parse(date)
                        if date else None for date in lei_results.lei_relationships_2_parent_last_updated
                    ],
                    "Relationship_2_Registration_Status": lei_results.lei_relationships_2_parent_registration_status,
                    "Relationship_2_Next_Renewal_Date": [
                        parse(date)
                        if date else None for date in lei_results.lei_relationships_2_parent_next_renewal_date
                    ],
                    "Relationship_2_Managing_Lou": lei_results.lei_relationships_2_parent_managing_LOU,
                    "Relationship_2_Validation_Sources": lei_results.lei_relationships_2_parent_validation_sources,
                    "Relationship_2_Validation_Documents": lei_results.lei_relationships_2_parent_validation_documents,
                    "Relationship_2_Validation_Reference": lei_results.lei_relationships_2_parent_validation_reference,
                    "Reporting_Exceptions_1_LEI": lei_results.lei_reporting_exception_1_lei,
                    "Reporting_Exceptions_1_Category": lei_results.lei_reporting_exception_1_category,
                    "Reporting_Exceptions_2_LEI": lei_results.lei_reporting_exception_2_lei,
                    "Reporting_Exceptions_2_Category": lei_results.lei_reporting_exception_2_category,
                }
            )

            return raw_output, lei_results, dataframe

        return raw_output, lei_results

    def _request_api(self, lei_list):
        """
        Requests LEI api and obtain the response

        Args:
            lei_list: List of LEIs
        
        Returns:
            api_responses: List of Dicts, json response from api
        """
        api_responses = []

        for lei in lei_list:
            try:
                response = requests.get(f"{self.api_url}/{lei}")
            except requests.exceptions.ConnectionError as err:
                logger.error("Error connecting to API host", exc_info=err)
                continue

            if not response.ok:
                logger.warning(f"No response from API for LEI number {lei}")
                continue

            api_responses.append(response.json())

        return api_responses
